@@ -38,8 +38,8 @@
     #include <unistd.h>
 #endif
 
-/* On Mac OS X send debug msg also to syslog for gui app in debug mode. */
-#if defined(__APPLE__) && defined(WINDOWED) && defined(LAUNCH_DEBUG)
+/* On Mac OS X send debug msg also to syslog for windowed app. */
+#if defined(__APPLE__) && defined(WINDOWED)
     #include <syslog.h>
 #endif
 
@@ -107,32 +107,46 @@ mbothererror(const char *fmt, ...)
     show_message_box(msg, "Error detected", MB_ICONWARNING);
 }
 
-    void mbfatal_winerror(const char * funcname, const char *fmt, ...)
-    {
-        char fullmsg[MBTXTLEN];
-        char msg[MBTXTLEN];
-        DWORD error_code = GetLastError();
-        va_list args;
+void mbfatal_winerror(const char * funcname, const char *fmt, ...)
+{
+    char fullmsg[MBTXTLEN];
+    char msg[MBTXTLEN];
+    DWORD error_code = GetLastError();
+    va_list args;
 
-        va_start(args, fmt);
-            vsnprintf(msg, MBTXTLEN, fmt, args);
-        va_end(args);
-        snprintf(fullmsg, MBTXTLEN, "%s%s: %s", msg, funcname, GetWinErrorString(error_code));
-        show_message_box(fullmsg, "Fatal error detected", MB_ICONEXCLAMATION);
-    }
+    va_start(args, fmt);
+    vsnprintf(msg, MBTXTLEN, fmt, args);
+    va_end(args);
 
-    void mbfatal_perror(const char * funcname, const char *fmt, ...)
-    {
-        char fullmsg[MBTXTLEN];
-        char msg[MBTXTLEN];
-        va_list args;
+    /* Suppress warnings generated by some mingw64 gcc toolchains;
+     * we do not care about truncation here. */
+    #pragma GCC diagnostic push
+    #pragma GCC diagnostic ignored "-Wformat-truncation"
+    snprintf(fullmsg, MBTXTLEN, "%s%s: %s", msg, funcname, GetWinErrorString(error_code));
+    #pragma GCC diagnostic pop
 
-        va_start(args, fmt);
-            vsnprintf(msg, MBTXTLEN, fmt, args);
-        va_end(args);
-        snprintf(fullmsg, MBTXTLEN, "%s%s: %s", msg, funcname, strerror(errno));
-        show_message_box(fullmsg, "Fatal error detected", MB_ICONEXCLAMATION);
-    }
+    show_message_box(fullmsg, "Fatal error detected", MB_ICONEXCLAMATION);
+}
+
+void mbfatal_perror(const char * funcname, const char *fmt, ...)
+{
+    char fullmsg[MBTXTLEN];
+    char msg[MBTXTLEN];
+    va_list args;
+
+    va_start(args, fmt);
+    vsnprintf(msg, MBTXTLEN, fmt, args);
+    va_end(args);
+
+    /* Suppress warnings generated by some mingw64 gcc toolchains;
+     * we do not care about truncation here. */
+    #pragma GCC diagnostic push
+    #pragma GCC diagnostic ignored "-Wformat-truncation"
+    snprintf(fullmsg, MBTXTLEN, "%s%s: %s", msg, funcname, strerror(errno));
+    #pragma GCC diagnostic pop
+
+    show_message_box(fullmsg, "Fatal error detected", MB_ICONEXCLAMATION);
+}
 #endif  /* _WIN32 and WINDOWED */
 
 /* Enable or disable debug output. */
@@ -204,7 +218,7 @@ pyi_global_printf(const char *fmt, ...)
     /* This allows to see bootloader debug messages in the Console.app log viewer. */
     /* https://en.wikipedia.org/wiki/Console_(OS_X) */
     /* Levels DEBUG and INFO are ignored so use level NOTICE. */
-#if defined(__APPLE__) && defined(WINDOWED) && defined(LAUNCH_DEBUG)
+#if defined(__APPLE__) && defined(WINDOWED)
     va_start(v, fmt);
     vsyslog(LOG_NOTICE, fmt, v);
     va_end(v);
@@ -223,7 +237,7 @@ void pyi_global_perror(const char *funcname, const char *fmt, ...) {
     va_end(v);
     perror(funcname);  // perror() writes to stderr
 
-    #if defined(__APPLE__) && defined(WINDOWED) && defined(LAUNCH_DEBUG)
+    #if defined(__APPLE__) && defined(WINDOWED)
         va_start(v, fmt);
             vsyslog(LOG_NOTICE, fmt, v);
             vsyslog(LOG_NOTICE, "%m\n", NULL);  // %m emits the result of strerror()
