@@ -18,6 +18,7 @@
 #define PYI_LAUNCH_H
 
 #include "pyi_archive.h"
+#include "pyi_splash.h"
 
 /*****************************************************************
 * The following 4 entries are for applications which may need to
@@ -25,14 +26,19 @@
 *****************************************************************/
 
 /*
- * Extract binaries in the archive
+ * Extract binaries from the archive
  *
- * @param workpath     (OUT) Where the binaries were extracted to. If
- *                      none extracted, is NULL.
+ * @param archive_status     The archive from which the binaries will
+ *                           be extracted.
+ *
+ * @param splash_status      The splash screen status which describes
+ *                           on which progress updated to occur. May
+ *                           be NULL
  *
  * @return 0 on success, non-zero otherwise.
  */
-int pyi_launch_extract_binaries(ARCHIVE_STATUS *archive_status);
+int pyi_launch_extract_binaries(ARCHIVE_STATUS *archive_status,
+                                SPLASH_STATUS *splash_status);
 
 /*
  * Check if binaries need to be extracted. If not, this is probably a onedir
@@ -64,21 +70,5 @@ int pyi_launch_execute(ARCHIVE_STATUS *status);
  */
 void pyi_parent_to_background();
 
-/*
- * Call a simple "int func(void)" entry point.  Assumes such a function
- * exists in the main namespace.
- * Return non zero on failure, with -2 if the specific error is
- * that the function does not exist in the namespace.
- *
- * @param name		Name of the function to execute.
- * @param presult	Integer return value.
- */
-int callSimpleEntryPoint(char *name, int *presult);
-
-/**
- * Clean up extracted binaries
- */
-void cleanUp(ARCHIVE_STATUS *status);
-
 #endif  /* PYI_LAUNCH_H */
 
