@@ -17,7 +17,6 @@
 
 #ifdef _WIN32
     #include <windows.h>  /* HMODULE */
-    #include <winsock.h>  /* ntohl */
 #else
     #include <dlfcn.h>  /* dlsym */
 #endif
@@ -40,6 +39,7 @@ DECLVAR(Py_NoSiteFlag);
 DECLVAR(Py_NoUserSiteDirectory);
 DECLVAR(Py_OptimizeFlag);
 DECLVAR(Py_VerboseFlag);
+DECLVAR(Py_UnbufferedStdioFlag);
 
 /* functions with prefix `Py_` */
 DECLPROC(Py_BuildValue);
@@ -86,6 +86,8 @@ DECLPROC(PyUnicode_FromFormat);
 DECLPROC(PyUnicode_DecodeFSDefault);
 DECLPROC(PyUnicode_Decode);
 DECLPROC(PyUnicode_AsUTF8);
+DECLPROC(PyUnicode_Join);
+DECLPROC(PyUnicode_Replace);
 
 DECLPROC(PyEval_EvalCode);
 DECLPROC(PyMarshal_ReadObjectFromString);
@@ -105,6 +107,7 @@ pyi_python_map_names(HMODULE dll, int pyvers)
     GETVAR(dll, Py_NoUserSiteDirectory);
     GETVAR(dll, Py_OptimizeFlag);
     GETVAR(dll, Py_VerboseFlag);
+    GETVAR(dll, Py_UnbufferedStdioFlag);
 
     /* functions with prefix `Py_` */
     GETPROC(dll, Py_BuildValue);
@@ -157,6 +160,8 @@ pyi_python_map_names(HMODULE dll, int pyvers)
     GETPROC(dll, PyUnicode_Decode);
     GETPROC(dll, PyUnicode_DecodeFSDefault);
     GETPROC(dll, PyUnicode_AsUTF8);
+    GETPROC(dll, PyUnicode_Join);
+    GETPROC(dll, PyUnicode_Replace);
 
     VS("LOADER: Loaded functions from Python library.\n");
 
