@@ -27,32 +27,6 @@
 #endif
 #include <wchar.h>
 
-/*
- * These macros used to define variables to hold dynamically accessed entry
- * points. These are declared 'extern' in this header, and defined fully later.
- */
-#ifdef _WIN32
-
-    #define EXTDECLPROC(result, name, args) \
-    typedef result (__cdecl *__PROC__ ## name) args; \
-    extern __PROC__ ## name PI_ ## name;
-
-    #define EXTDECLVAR(vartyp, name) \
-    typedef vartyp __VAR__ ## name; \
-    extern __VAR__ ## name *PI_ ## name;
-
-#else
-
-    #define EXTDECLPROC(result, name, args) \
-    typedef result (*__PROC__ ## name) args; \
-    extern __PROC__ ## name PI_ ## name;
-
-    #define EXTDECLVAR(vartyp, name) \
-    typedef vartyp __VAR__ ## name; \
-    extern __VAR__ ## name *PI_ ## name;
-
-#endif  /* WIN32 */
-
 /*
  * Python.h replacements.
  *
@@ -89,6 +63,7 @@ EXTDECLVAR(int, Py_VerboseFlag);
 EXTDECLVAR(int, Py_IgnoreEnvironmentFlag);
 EXTDECLVAR(int, Py_DontWriteBytecodeFlag);
 EXTDECLVAR(int, Py_NoUserSiteDirectory);
+EXTDECLVAR(int, Py_UnbufferedStdioFlag);
 
 /* This initializes the table of loaded modules (sys.modules), and creates the fundamental modules builtins, __main__ and sys. It also initializes the module search path (sys.path). It does not set sys.argv; */
 EXTDECLPROC(int, Py_Initialize, (void));
@@ -159,88 +134,8 @@ EXTDECLPROC(void, PyErr_Restore, (PyObject *, PyObject *, PyObject *));
 EXTDECLPROC(PyObject *, PyObject_Str, (PyObject *));
 EXTDECLPROC(PyObject *, PyObject_GetAttrString, (PyObject *, const char *));
 EXTDECLPROC(const char *, PyUnicode_AsUTF8, (PyObject *));
-
-/*
- * Macros for reference counting through exported functions
- * (that is: without binding to the binary structure of a PyObject.
- * These rely on the Py_IncRef/Py_DecRef API functions on Pyhton 2.4+.
- *
- * Python versions before 2.4 do not export IncRef/DecRef as a binary API,
- * but only as macros in header files. Since we support Python 2.4+ we do not
- * need to provide an emulated incref/decref as it was with older Python
- * versions.
- *
- * We do not want to depend on Python.h for many reasons (including the fact
- * that we would like to have a single binary for all Python versions).
- */
-
-#define Py_XINCREF(o)    PI_Py_IncRef(o)
-#define Py_XDECREF(o)    PI_Py_DecRef(o)
-#define Py_DECREF(o)     Py_XDECREF(o)
-#define Py_INCREF(o)     Py_XINCREF(o)
-
-/* Macros to declare and get Python entry points in the C file.
- * Typedefs '__PROC__...' have been done above
- *
- * GETPROC_RENAMED is to support Python APIs that are simply renamed. We use
- * the new name, and when loading an old Python lib, load the old symbol into the
- * new name.
- */
-#ifdef _WIN32
-
-    #define DECLPROC(name) \
-    __PROC__ ## name PI_ ## name = NULL;
-    #define GETPROCOPT(dll, name, sym) \
-    PI_ ## name = (__PROC__ ## name)GetProcAddress (dll, #sym)
-    #define GETPROC(dll, name) \
-    GETPROCOPT(dll, name, name); \
-    if (!PI_ ## name) { \
-        FATAL_WINERROR("GetProcAddress", "Failed to get address for " #name "\n");\
-        return -1; \
-    }
-    #define GETPROC_RENAMED(dll, name, sym) \
-    GETPROCOPT(dll, name, sym); \
-    if (!PI_ ## name) { \
-        FATAL_WINERROR("GetProcAddress", "Failed to get address for " #sym "\n");\
-        return -1; \
-    }
-    #define DECLVAR(name) \
-    __VAR__ ## name * PI_ ## name = NULL;
-    #define GETVAR(dll, name) \
-    PI_ ## name = (__VAR__ ## name *)GetProcAddress (dll, #name); \
-    if (!PI_ ## name) { \
-        FATAL_WINERROR("GetProcAddress", "Failed to get address for " #name "\n");\
-        return -1; \
-    }
-
-#else /* ifdef _WIN32 */
-
-    #define DECLPROC(name) \
-    __PROC__ ## name PI_ ## name = NULL;
-    #define GETPROCOPT(dll, name, sym) \
-    PI_ ## name = (__PROC__ ## name)dlsym (dll, #sym)
-    #define GETPROC(dll, name) \
-    GETPROCOPT(dll, name, name); \
-    if (!PI_ ## name) { \
-        FATALERROR ("Cannot dlsym for " #name "\n"); \
-        return -1; \
-    }
-    #define GETPROC_RENAMED(dll, name, sym) \
-    GETPROCOPT(dll, name, sym); \
-    if (!PI_ ## name) { \
-        FATALERROR ("Cannot dlsym for " #sym "\n"); \
-        return -1; \
-    }
-    #define DECLVAR(name) \
-    __VAR__ ## name * PI_ ## name = NULL;
-    #define GETVAR(dll, name) \
-    PI_ ## name = (__VAR__ ## name *)dlsym(dll, #name); \
-    if (!PI_ ## name) { \
-        FATALERROR ("Cannot dlsym for " #name "\n"); \
-        return -1; \
-    }
-
-#endif  /* WIN32 */
+EXTDECLPROC(PyObject *, PyUnicode_Join, (PyObject *, PyObject *));
+EXTDECLPROC(PyObject *, PyUnicode_Replace, (PyObject *, PyObject *, PyObject *, size_t));  /* Py_ssize_t */
 
 int pyi_python_map_names(HMODULE dll, int pyvers);
 
