@@ -21,15 +21,8 @@
     #include <windows.h> /* HMODULE */
     #include <fcntl.h>   /* O_BINARY */
     #include <io.h>      /* _setmode */
-    #include <winsock.h> /* ntohl */
 #else
     #include <dlfcn.h>  /* dlerror */
-    #ifdef __FreeBSD__
-/* freebsd issue #188316 */
-        #include <arpa/inet.h>  /* ntohl */
-    #else
-        #include <netinet/in.h>  /* ntohl */
-    #endif
     #include <stdlib.h>  /* mbstowcs */
 #endif /* ifdef _WIN32 */
 #include <stddef.h>  /* ptrdiff_t */
@@ -78,11 +71,11 @@ pyi_pylib_load(ARCHIVE_STATUS *status)
       uint32_t pyvers_major;
       uint32_t pyvers_minor;
 
-      pyvers_major = pyvers / 10;
-      pyvers_minor = pyvers % 10;
+      pyvers_major = pyvers / 100;
+      pyvers_minor = pyvers % 100;
 
       len = snprintf(dllname, DLLNAME_LEN,
-              "libpython%01d.%01d.a(libpython%01d.%01d.so)",
+              "libpython%d.%d.a(libpython%d.%d.so)",
               pyvers_major, pyvers_minor, pyvers_major, pyvers_minor);
     }
     else {
@@ -157,8 +150,9 @@ pyi_pylib_attach(ARCHIVE_STATUS *status, int *loadedNew)
     HMODULE dll;
     char nm[PATH_MAX + 1];
     int ret = 0;
+
     /* Get python's name */
-    sprintf(nm, "python%02d.dll", pyvers);
+    sprintf(nm, "python%d%d.dll", pyvers / 100, pyvers % 100);
 
     /* See if it's loaded */
     dll = GetModuleHandleA(nm);
@@ -247,6 +241,9 @@ pyi_pylib_set_runtime_opts(ARCHIVE_STATUS *status)
         setbuf(stdin, (char *)NULL);
         setbuf(stdout, (char *)NULL);
         setbuf(stderr, (char *)NULL);
+
+        /* Enable unbuffered mode via Py_UnbufferedStdioFlag */
+        *PI_Py_UnbufferedStdioFlag = 1;
     }
     return 0;
 }
@@ -396,15 +393,16 @@ pyi_pylib_start_python(ARCHIVE_STATUS *status)
      * its contents nor free its memory.
      *
      * NOTE: Statics are zero-initialized. */
-    static char pypath[2 * PATH_MAX + 14];
+    #define MAX_PYPATH_SIZE (3 * PATH_MAX + 32)
+    static char pypath[MAX_PYPATH_SIZE];
 
     /* Wide string forms of the above, for Python 3. */
-    static wchar_t pypath_w[PATH_MAX + 1];
+    static wchar_t pypath_w[MAX_PYPATH_SIZE];
     static wchar_t pyhome_w[PATH_MAX + 1];
     static wchar_t progname_w[PATH_MAX + 1];
 
     /* Decode using current locale */
-    if (!pyi_locale_char2wchar(progname_w, status->archivename, PATH_MAX)) {
+    if (!pyi_locale_char2wchar(progname_w, status->executablename, PATH_MAX)) {
         FATALERROR("Failed to convert progname to wchar_t\n");
         return -1;
     }
@@ -423,23 +421,27 @@ pyi_pylib_start_python(ARCHIVE_STATUS *status)
     PI_Py_SetPythonHome(pyhome_w);
 
     /* Set sys.path */
-    /* sys.path = [base_library, mainpath] */
-    if (snprintf(pypath, sizeof pypath, "%s%cbase_library.zip%c%s",
-                 status->mainpath, PYI_SEP, PYI_PATHSEP, status->mainpath)
-        >= sizeof pypath) {
+    /* sys.path = [mainpath/base_library.zip, mainpath/lib-dynload, mainpath] */
+    if (snprintf(pypath, MAX_PYPATH_SIZE, "%s%c%s" "%c" "%s%c%s" "%c" "%s",
+                 status->mainpath, PYI_SEP, "base_library.zip",
+                 PYI_PATHSEP,
+                 status->mainpath, PYI_SEP, "lib-dynload",
+                 PYI_PATHSEP,
+                 status->mainpath)
+        >= MAX_PYPATH_SIZE) {
         // This should never happen, since mainpath is < PATH_MAX and pypath is
         // huge enough
         FATALERROR("sys.path (based on %s) exceeds buffer[%d] space\n",
-                   status->mainpath, sizeof pypath);
+                   status->mainpath, MAX_PYPATH_SIZE);
         return -1;
     }
 
     /*
-     * E must set sys.path to have base_library.zip before
+     * We must set sys.path to have base_library.zip before
      * calling Py_Initialize as it needs `encodings` and other modules.
      */
     /* Decode using current locale */
-    if (!pyi_locale_char2wchar(pypath_w, pypath, PATH_MAX)) {
+    if (!pyi_locale_char2wchar(pypath_w, pypath, MAX_PYPATH_SIZE)) {
         FATALERROR("Failed to convert pypath to wchar_t\n");
         return -1;
     }
@@ -497,7 +499,7 @@ pyi_pylib_start_python(ARCHIVE_STATUS *status)
 
     /* Check for a python error */
     if (PI_PyErr_Occurred()) {
-        FATALERROR("Error detected starting Python VM.");
+        FATALERROR("Error detected starting Python VM.\n");
         return -1;
     }
 
@@ -510,9 +512,6 @@ pyi_pylib_start_python(ARCHIVE_STATUS *status)
 int
 pyi_pylib_import_modules(ARCHIVE_STATUS *status)
 {
-    PyObject *marshal;
-    PyObject *marshaldict;
-    PyObject *loadfunc;
     TOC *ptoc;
     PyObject *co;
     PyObject *mod;
@@ -539,14 +538,6 @@ pyi_pylib_import_modules(ARCHIVE_STATUS *status)
 
     VS("LOADER: importing modules from CArchive\n");
 
-    /* Get the Python function marshall.load
-     * Here we collect some reference to PyObject that we don't dereference
-     * Doesn't matter because the objects won't be going away anyway.
-     */
-    marshal = PI_PyImport_ImportModule("marshal");
-    marshaldict = PI_PyModule_GetDict(marshal);
-    loadfunc = PI_PyDict_GetItemString(marshaldict, "loads");
-
     /* Iterate through toc looking for module entries (type 'm')
      * this is normally just bootstrap stuff (archive and iu)
      */
@@ -559,35 +550,27 @@ pyi_pylib_import_modules(ARCHIVE_STATUS *status)
 
             VS("LOADER: extracted %s\n", ptoc->name);
 
-            /* .pyc/.pyo files have 8 bytes header. Skip it and load marshalled
-             * data form the right point.
-             */
-            if (pyvers >= 37) {
-                /* Python >= 3.7 the header: size was changed to 16 bytes. */
-                co = PI_PyObject_CallFunction(loadfunc, "y#", modbuf + 16,
-                                              ntohl(ptoc->ulen) - 16);
+            /* Unmarshall code object for module; we need to skip
+               the pyc header */
+            if (pyvers >= 307) {
+                /* Python 3.7 changed header size to 16 bytes */
+                co = PI_PyMarshal_ReadObjectFromString((const char *) modbuf + 16, ptoc->ulen - 16);
+            } else {
+                co = PI_PyMarshal_ReadObjectFromString((const char *) modbuf + 12, ptoc->ulen - 12);
             }
-            else {
-                /* It looks like from python 3.3 the header */
-                /* size was changed to 12 bytes. */
-                co =
-                    PI_PyObject_CallFunction(loadfunc, "y#", modbuf + 12, ntohl(
-                                                 ptoc->ulen) - 12);
-            };
 
             if (co != NULL) {
-                VS("LOADER: callfunction returned...\n");
+                VS("LOADER: running unmarshalled code object for %s...\n", ptoc->name);
                 mod = PI_PyImport_ExecCodeModule(ptoc->name, co);
             }
             else {
-                /* TODO callfunctions might return NULL - find yout why and foor what modules. */
-                VS("LOADER: callfunction returned NULL");
+                VS("LOADER: failed to unmarshall code object for %s!\n", ptoc->name);
                 mod = NULL;
             }
 
             /* Check for errors in loading */
             if (mod == NULL) {
-                FATALERROR("mod is NULL - %s", ptoc->name);
+                FATALERROR("Module object for %s is NULL!\n", ptoc->name);
             }
 
             if (PI_PyErr_Occurred()) {
@@ -619,7 +602,7 @@ int
 pyi_pylib_install_zlib(ARCHIVE_STATUS *status, TOC *ptoc)
 {
     int rc = 0;
-    size_t zlibpos = status->pkgstart + ntohl(ptoc->pos);
+    uint64_t zlibpos = status->pkgstart + ptoc->pos;
     PyObject * sys_path, *zlib_entry, *archivename_obj;
 
     /* Note that sys.path contains PyUnicode on py3. Ensure
@@ -637,7 +620,7 @@ pyi_pylib_install_zlib(ARCHIVE_STATUS *status, TOC *ptoc)
      */
     archivename_obj = PI_PyUnicode_DecodeFSDefault(status->archivename);
 #endif
-    zlib_entry = PI_PyUnicode_FromFormat("%U?%zu", archivename_obj, zlibpos);
+    zlib_entry = PI_PyUnicode_FromFormat("%U?%" PRIu64, archivename_obj, zlibpos);
     PI_Py_DecRef(archivename_obj);
 
     sys_path = PI_PySys_GetObject("path");
@@ -693,7 +676,7 @@ pyi_pylib_finalize(ARCHIVE_STATUS *status)
      */
     if (status->is_pylib_loaded == true) {
         #ifndef WINDOWED
-            /* 
+            /*
              * We need to manually flush the buffers because otherwise there can be errors.
              * The native python interpreter flushes buffers before calling Py_Finalize,
              * so we need to manually do the same. See isse #4908.
