@@ -93,18 +93,6 @@ typedef void (*sighandler_t)(int);
 static char **argv_pyi = NULL;
 static int argc_pyi = 0;
 
-/*
- * Watch for OpenDocument AppleEvents and add the files passed in to the
- * sys.argv command line on the Python side.
- *
- * This allows on Mac OS X to open files when a file is dragged and dropped
- * on the App icon in the OS X dock.
- */
-#if defined(__APPLE__) && defined(WINDOWED)
-static void process_apple_events(Boolean);
-#endif
-
-
 // some platforms do not provide strnlen
 #ifndef HAVE_STRNLEN
 size_t
@@ -588,19 +576,6 @@ pyi_remove_temp_path(const char *dir)
 }
 #endif /* ifdef _WIN32 */
 
-/* TODO is this function still used? Could it be removed? */
-/*
- * If binaries were extracted, this should be called
- * to remove them
- */
-void
-cleanUp(ARCHIVE_STATUS *status)
-{
-    if (status->temppath[0]) {
-        pyi_remove_temp_path(status->temppath);
-    }
-}
-
 /*
  * helper for extract2fs
  * which may try multiple places
@@ -723,7 +698,6 @@ pyi_copy_file(const char *src, const char *dst, const char *filename)
     return error;
 }
 
-/* TODO use dlclose() when exiting. */
 /* Load the shared dynamic library (DLL) */
 dylib_t
 pyi_utils_dlopen(const char *dllpath)
@@ -754,6 +728,26 @@ pyi_utils_dlopen(const char *dllpath)
 
 }
 
+/* TODO use pyi_utils_dlclose() when exiting. */
+/* Unlink/Close the shared library.
+ * Returns zero on success, a nonzero value on failure.
+ *
+ * Interesting fact: many debuggers link to attached libraries
+ * too, therefore calling dlclose from within the bootloader
+ * does **not** necessarily mean the library will be unloaded
+ * if a debugger is attached. */
+int
+pyi_utils_dlclose(dylib_t dll)
+{
+#ifdef _WIN32
+    /* FreeLibrary returns a nonzero value on success,
+     * invert it to provide a common return value */
+    return !FreeLibrary(dll);
+#else
+    return dlclose(dll);
+#endif
+}
+
 /* ////////////////////////////////////////////////////////////////// */
 /* TODO better merging of the following platform specific functions. */
 /* ////////////////////////////////////////////////////////////////// */
@@ -984,36 +978,13 @@ pyi_utils_create_child(const char *thisfile, const ARCHIVE_STATUS* status,
     int ignore_signals;
     int signum;
 
-    argv_pyi = (char**)calloc(argc + 1, sizeof(char*));
-    argc_pyi = 0;
-    if (!argv_pyi) {
-        FATALERROR("LOADER: failed to allocate argv_pyi: %s\n", strerror(errno));
+    /* Initialize argv_pyi and argc_pyi */
+    if (pyi_utils_initialize_args(argc, argv) < 0) {
         goto cleanup;
     }
 
-    for (i = 0; i < argc; i++) {
     #if defined(__APPLE__) && defined(WINDOWED)
-
-        /* if we are on a Mac, it passes a strange -psnxxx argument.  Filter it out. */
-        if (strstr(argv[i], "-psn") == argv[i]) {
-            /* skip */
-        }
-        else
-    #endif
-        {
-            char *const tmp = strdup(argv[i]);
-            if (!tmp) {
-                FATALERROR("LOADER: failed to strdup argv[%d]: %s\n", i, strerror(errno));
-                /* If we can't allocate basic amounts of memory at this critical point,
-                 * we should probably just give up. */
-                goto cleanup;
-            }
-            argv_pyi[argc_pyi++] = tmp;
-        }
-    }
-
-    #if defined(__APPLE__) && defined(WINDOWED)
-    process_apple_events(true /* short timeout (250 ms) */);
+    pyi_process_apple_events(true /* short timeout (250 ms) */);
     #endif
 
     pid = fork();
@@ -1068,7 +1039,7 @@ pyi_utils_create_child(const char *thisfile, const ARCHIVE_STATUS* status,
         if (wait_rc == 0) {
             /* Child not done yet -- wait for and process AppleEvents with a
              * 1 second timeout, forwarding file-open events to the child. */
-            process_apple_events(false /* long timeout (1 sec) */);
+            pyi_process_apple_events(false /* long timeout (1 sec) */);
         }
     } while (!wait_rc);
     #else
@@ -1086,10 +1057,7 @@ pyi_utils_create_child(const char *thisfile, const ARCHIVE_STATUS* status,
 
   cleanup:
     VS("LOADER: freeing args\n");
-    for (i = 0; i < argc_pyi; i++) {
-        free(argv_pyi[i]);
-    }
-    free(argv_pyi);
+    pyi_utils_free_args();
 
     /* Either wait() failed, or we jumped to `cleanup` and
      * didn't wait() at all. Either way, exit with error,
@@ -1113,6 +1081,97 @@ pyi_utils_create_child(const char *thisfile, const ARCHIVE_STATUS* status,
     return 1;
 }
 
+
+/*
+ * Initialize private argc_pyi and argv_pyi from the given argc and
+ * argv by creating a deep copy. The resulting argc_pyi and argv_pyi
+ * can be retrieved by pyi_utils_get_args() and are freed/cleaned-up by
+ * pyi_utils_free_args().
+ *
+ * The argv_pyi contains argc_pyi + 1 elements, with the last element
+ * being NULL (i.e., it is execv-compatible NULL-terminated array).
+ *
+ * On macOS, this function filters out the -psnxxx argument that is
+ * passed to executable when .app bundle is launched from Finder:
+ * https://stackoverflow.com/questions/10242115/os-x-strange-psn-command-line-parameter-when-launched-from-finder
+ */
+int pyi_utils_initialize_args(const int argc, char *const argv[])
+{
+    int i;
+
+    argv_pyi = (char**)calloc(argc + 1, sizeof(char*));
+    argc_pyi = 0;
+    if (!argv_pyi) {
+        FATALERROR("LOADER: failed to allocate argv_pyi: %s\n", strerror(errno));
+        return -1;
+    }
+
+    for (i = 0; i < argc; i++) {
+        char *tmp;
+
+        /* Filter out -psnxxx argument that is used on macOS to pass
+         * unique process serial number (PSN) to apps launched via Finder. */
+        #if defined(__APPLE__) && defined(WINDOWED)
+        if (strstr(argv[i], "-psn") == argv[i]) {
+            continue;
+        }
+        #endif
+
+        /* Copy the argument */
+        tmp = strdup(argv[i]);
+        if (!tmp) {
+            FATALERROR("LOADER: failed to strdup argv[%d]: %s\n", i, strerror(errno));
+            /* If we can't allocate basic amounts of memory at this critical point,
+             * we should probably just give up. */
+            return -1;
+        }
+        argv_pyi[argc_pyi++] = tmp;
+    }
+
+    return 0;
+}
+
+/*
+ * Retrieve value of argc_pyi and the pointer to argv_pyi. The retrieved
+ * arguments are originally the same as the ones passed to
+ * pyi_utils_initialize_args(), but may have been modified by subsequent
+ * processing code (e.g., Apple event processing).
+ *
+ * The argv_pyi array is NULL terminated (i.e., contains argc_pyi + 1)
+ * entries, and the last entry is NULL).
+ *
+ * The ownership of array is not transferred, i.e., it should not be
+ * explicitly freed by the caller. Instead, the array and its resources
+ * are cleaned up oncepyi_utils_free_args() is called.
+ */
+void pyi_utils_get_args(int *argc, char ***argv)
+{
+    if (argc) {
+        *argc = argc_pyi;
+    }
+    if (argv) {
+        *argv = argv_pyi;
+    }
+}
+
+/*
+ * Free/clean-up the private arguments (pyi_argv).
+ */
+void pyi_utils_free_args()
+{
+    /* Free each entry */
+    int i;
+    for (i = 0; i < argc_pyi; i++) {
+        free(argv_pyi[i]);
+    }
+    /* Free the list */
+    free(argv_pyi);
+    /* Clean-up the variables, just in case */
+    argc_pyi = 0;
+    argv_pyi = NULL;
+}
+
+
 /*
  * On Mac OS X this converts events from kAEOpenDocuments and kAEGetURL into sys.argv.
  * After startup, it also forwards kAEOpenDocuments and KAEGetURL events at runtime to the child process.
@@ -1406,8 +1465,8 @@ static pascal OSErr handle_apple_event(const AppleEvent *theAppleEvent, AppleEve
         return generic_forward_apple_event(theAppleEvent, kAEMiscStandards, evtID, "Activate");
     default:
         /* Not 'GURL', 'odoc', 'rapp', or 'actv'  -- this is not reached unless there is a
-         * programming error in the code that sets up the handler(s) in process_apple_events. */
-        OTHERERROR("LOADER [AppleEvent]: %s called with unexpected event type '%s'!",
+         * programming error in the code that sets up the handler(s) in pyi_process_apple_events. */
+        OTHERERROR("LOADER [AppleEvent]: %s called with unexpected event type '%s'!\n",
                    __FUNCTION__, CC2Str(evtCode));
         return errAEEventNotHandled;
     }
@@ -1437,7 +1496,7 @@ static OSStatus evt_handler_proc(EventHandlerCallRef href, EventRef eref, void *
     VS("LOADER [AppleEvent]: what=%hu message=%lx ('%s') modifiers=%hu\n",
        eventRecord.what, eventRecord.message, CC2Str((FourCharCode)eventRecord.message), eventRecord.modifiers);
     /* This will end up calling one of the callback functions
-     * that we installed in process_apple_events() */
+     * that we installed in pyi_process_apple_events() */
     err = AEProcessAppleEvent(&eventRecord);
     if (err == errAEEventNotHandled) {
         VS("LOADER [AppleEvent]: Ignored event.\n");
@@ -1451,7 +1510,7 @@ static OSStatus evt_handler_proc(EventHandlerCallRef href, EventRef eref, void *
 }
 
 /* Apple event message pump */
-static void process_apple_events(Boolean short_timeout)
+void pyi_process_apple_events(bool short_timeout)
 {
     static EventHandlerUPP handler;
     static AEEventHandlerUPP handler_ae;
@@ -1541,7 +1600,7 @@ static void process_apple_events(Boolean short_timeout)
         static Boolean once = false;
         if (!once) {
             /* Log this only once since this is compiled-in even in non-debug mode and we
-             * want to avoid console spam, since process_apple_events may be called a lot. */
+             * want to avoid console spam, since pyi_process_apple_events may be called a lot. */
             OTHERERROR("LOADER [AppleEvent]: ERROR installing handler.\n");
             once = true;
         }
