@@ -33,18 +33,10 @@
 #include <sddl.h>     /* ConvertStringSecurityDescriptorToSecurityDescriptorW */
 
 /* PyInstaller headers. */
-#include "msvc_stdint.h" /* int32_t */
 #include "pyi_global.h"  /* PATH_MAX */
 #include "pyi_utils.h"
 #include "pyi_win32_utils.h"
 
-static HANDLE hCtx = INVALID_HANDLE_VALUE;
-static ULONG_PTR actToken;
-
-#ifndef STATUS_SXS_EARLY_DEACTIVATION
-    #define STATUS_SXS_EARLY_DEACTIVATION 0xC015000F
-#endif
-
 #define ERROR_STRING_MAX 4096
 static char errorString[ERROR_STRING_MAX];
 
@@ -93,53 +85,6 @@ char * GetWinErrorString(DWORD error_code) {
     return errorString;
 }
 
-int
-CreateActContext(const char *manifestpath)
-{
-    wchar_t * manifestpath_w;
-    ACTCTXW ctx;
-    BOOL activated;
-    HANDLE k32;
-
-    HANDLE (WINAPI * CreateActCtx)(PACTCTXW pActCtx);
-    BOOL (WINAPI * ActivateActCtx)(HANDLE hActCtx, ULONG_PTR * lpCookie);
-
-    /* Setup activation context */
-    VS("LOADER: manifestpath: %s\n", manifestpath);
-    manifestpath_w = pyi_win32_utils_from_utf8(NULL, manifestpath, 0);
-
-    k32 = LoadLibraryA("kernel32");
-    CreateActCtx = (void*)GetProcAddress(k32, "CreateActCtxW");
-    ActivateActCtx = (void*)GetProcAddress(k32, "ActivateActCtx");
-
-    if (!CreateActCtx || !ActivateActCtx) {
-        VS("LOADER: Cannot find CreateActCtx/ActivateActCtx exports in kernel32.dll\n");
-        return 0;
-    }
-
-    ZeroMemory(&ctx, sizeof(ctx));
-    ctx.cbSize = sizeof(ACTCTX);
-    ctx.lpSource = manifestpath_w;
-    ctx.dwFlags = ACTCTX_FLAG_SET_PROCESS_DEFAULT;
-
-    hCtx = CreateActCtx(&ctx);
-    free(manifestpath_w);
-
-    if (hCtx != INVALID_HANDLE_VALUE) {
-        VS("LOADER: Activation context created\n");
-        activated = ActivateActCtx(hCtx, &actToken);
-
-        if (activated) {
-            VS("LOADER: Activation context activated\n");
-            return 1;
-        }
-    }
-
-    hCtx = INVALID_HANDLE_VALUE;
-    VS("LOADER: Error activating the context: ActivateActCtx: \n%s\n", GetWinErrorString(0));
-    return 0;
-}
-
 /* Convert a wide string to an ANSI string.
  *
  *  Returns a newly allocated buffer containing the ANSI characters terminated by a null
@@ -176,7 +121,7 @@ pyi_win32_wcs_to_mbs(const wchar_t *wstr)
 
     str = (char *)calloc(len + 1, sizeof(char));
     if (str == NULL) {
-        FATAL_WINERROR("win32_wcs_to_mbs", "Out of memory.");
+        FATAL_WINERROR("win32_wcs_to_mbs", "Out of memory.\n");
         return NULL;
     };
 
@@ -311,7 +256,7 @@ pyi_win32_utils_to_utf8(char *str, const wchar_t *wstr, size_t len)
 
         output = (char *)calloc(len + 1, sizeof(char));
         if (output == NULL) {
-            FATAL_WINERROR("win32_utils_to_utf8", "Out of memory.");
+            FATAL_WINERROR("win32_utils_to_utf8", "Out of memory.\n");
             return NULL;
         };
     }
@@ -375,7 +320,7 @@ pyi_win32_utils_from_utf8(wchar_t *wstr, const char *str, size_t wlen)
 
         output = (wchar_t *)calloc(wlen + 1, sizeof(wchar_t));
         if (output == NULL) {
-            FATAL_WINERROR("win32_utils_from_utf8", "Out of memory.");
+            FATAL_WINERROR("win32_utils_from_utf8", "Out of memory.\n");
             return NULL;
         };
     }
